package com.ctgu.pro_eld_mqtt_compare.api;

import com.ctgu.pro_eld_mqtt_compare.request.GetInfoVO;
import com.ctgu.pro_eld_mqtt_compare.response.*;
import com.ctgu.pro_eld_mqtt_compare.utils.TAG;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Log4j2
@Component
public class ApiService {

    @Autowired
    WebClient.Builder webClientBuilder;

    private static final Map<String, ParameterizedTypeReference<?>> tableObjectMapping = new HashMap<>();

    static {
        tableObjectMapping.put(TAG.VEHICLE, new ParameterizedTypeReference<List<VehicleDTO>>() {
        });
        tableObjectMapping.put(TAG.DEVICE, new ParameterizedTypeReference<List<DeviceDTO>>() {
        });
        tableObjectMapping.put(TAG.BUSINESS, new ParameterizedTypeReference<List<BusinessDTO>>() {
        });
        tableObjectMapping.put(TAG.DRIVER, new ParameterizedTypeReference<List<DriverDTO>>() {
        });
        tableObjectMapping.put(TAG.VEHICLE_DEVICE, new ParameterizedTypeReference<List<VehicleDeviceDTO>>() {
        });
        tableObjectMapping.put(TAG.DRIVER_VEHICLE, new ParameterizedTypeReference<List<DriverVehicleDTO>>() {
        });
        tableObjectMapping.put(TAG.BUSINESS_DRIVER, new ParameterizedTypeReference<List<BusinessDriverDTO>>() {
        });
    }

    // Pakistan API
    @SuppressWarnings("unchecked")
    public <T> Mono<List<T>> getInfoPakistan(GetInfoVO getInfoVO) {
        ParameterizedTypeReference<List<T>> typeRef = (ParameterizedTypeReference<List<T>>) tableObjectMapping.get(getInfoVO.getTableName());
        if (typeRef == null) {
            throw new IllegalArgumentException("不支持该表名: " + getInfoVO.getTableName());
        }

        return webClientBuilder
                .baseUrl("http://34.81.61.186")
                .build()
                .post()
                .uri("/api/eldapp/v1/eldData")
                .header("eldKey", "anytrek-eld-super-key-2024")
                .contentType(MediaType.APPLICATION_JSON)
                .bodyValue(getInfoVO)
                .retrieve()
                .bodyToMono(typeRef);
    }

    // Pro API
    @SuppressWarnings("unchecked")
    public <T> Mono<List<T>> getInfoPro(Integer customerId, String tableName) {
        ParameterizedTypeReference<List<T>> typeRef = (ParameterizedTypeReference<List<T>>) tableObjectMapping.get(tableName);
        if (typeRef == null) {
            throw new IllegalArgumentException("Unsupported table name: " + tableName);
        }

        return webClientBuilder.baseUrl("https://eldapp.anytrek.app")
                .build()
                .get()
                .uri(uriBuilder -> uriBuilder
                        .path("/api/eldapp/v1/eldData")
                        .queryParam("customerId", customerId)
                        .queryParam("tableName", tableName)
                        .build())
                .header("eldKey", "435e833ca0610442d25b8011475c8352")
                .retrieve()
                .bodyToMono(typeRef);
    }
}