package com.ctgu.pro_eld_mqtt_compare.mqtt;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.integration.annotation.ServiceActivator;
import org.springframework.integration.channel.DirectChannel;
import org.springframework.integration.mqtt.core.DefaultMqttPahoClientFactory;
import org.springframework.integration.mqtt.core.MqttPahoClientFactory;
import org.springframework.integration.mqtt.inbound.MqttPahoMessageDrivenChannelAdapter;
import org.springframework.integration.mqtt.support.DefaultPahoMessageConverter;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.MessageHandler;

import java.util.UUID;

@Configuration
public class MqttInboundConfiguration {

    private final MqttInboundProperties mqttProperties;
    private final ReceiverMessageHandler receiverMessageHandler;

    @Autowired
    public MqttInboundConfiguration(MqttInboundProperties mqttProperties, ReceiverMessageHandler receiverMessageHandler) {
        this.mqttProperties = mqttProperties;
        this.receiverMessageHandler = receiverMessageHandler;
    }

    @Bean
    public MqttPahoClientFactory mqttInboundClientFactory() {
        DefaultMqttPahoClientFactory factory = new DefaultMqttPahoClientFactory();
        factory.getConnectionOptions().setServerURIs(new String[]{mqttProperties.getUrl()});
        factory.getConnectionOptions().setUserName(mqttProperties.getUsername());
        factory.getConnectionOptions().setPassword(mqttProperties.getPassword().toCharArray());
        factory.getConnectionOptions().setKeepAliveInterval(30);
        factory.getConnectionOptions().setAutomaticReconnect(true);
        factory.getConnectionOptions().setCleanSession(false);
        return factory;
    }

    @Bean
    public MessageChannel messageInboundChannel() {
        return new DirectChannel();
    }

    @Bean
    public MqttPahoMessageDrivenChannelAdapter mqttInboundAdapter() {
        return new MqttPahoMessageDrivenChannelAdapter(
                mqttProperties.getUrl(),
                mqttProperties.getClientId() + UUID.randomUUID(),
                mqttInboundClientFactory(),
                mqttProperties.getTopic()
        ) {{
            setQos(mqttProperties.getQos());
            setConverter(new DefaultPahoMessageConverter());
            setOutputChannel(messageInboundChannel());
        }};
    }

    @Bean
    @ServiceActivator(inputChannel = "messageInboundChannel")
    public MessageHandler messageHandler() {
        return receiverMessageHandler;
    }
}
