package com.ctgu.pro_eld_mqtt_compare.mqtt;


import com.ctgu.pro_eld_mqtt_compare.request.PublishMQTTVO;
import com.ctgu.pro_eld_mqtt_compare.response.MqttMessageDTO;
import com.ctgu.pro_eld_mqtt_compare.service.CompareService;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.integration.annotation.Publisher;
import org.springframework.integration.annotation.ServiceActivator;
import org.springframework.messaging.Message;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.MessageHandler;
import org.springframework.messaging.MessagingException;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;
import java.util.Set;

@Log4j2
@Component
public class ReceiverMessageHandler implements MessageHandler {

    private static final String TAG = ReceiverMessageHandler.class.getSimpleName();

    @Autowired
    @Qualifier("mqttOutboundChannel")
    private MessageChannel mqttOutboundChannel;

    private final CompareService compareService;

    public ReceiverMessageHandler(CompareService compareService) {
        this.compareService = compareService;
    }

    @SneakyThrows
    @ServiceActivator(inputChannel = "messageInboundChannel")
    public void handleMessage(Message<?> message) throws MessagingException {
        String mqttMessage = (String) message.getPayload();
        log.info("{} : <----- MQTT message = {}", TAG, mqttMessage);
        ObjectMapper objectMapper = new ObjectMapper();
        JsonNode rootNode = objectMapper.readTree(mqttMessage);

        // 处理对象类型
        if (rootNode.isObject()) {
            MqttMessageDTO messageDTO = objectMapper.treeToValue(rootNode, MqttMessageDTO.class);
            PublishMQTTVO compare = compareService.compare(messageDTO.getCustomerId(), messageDTO.getTableName());
            String json = objectMapper.writeValueAsString(compare);
            mqttOutboundChannel.send(MessageBuilder.withPayload(json).build());
            log.info("{} : -----> MQTT message = {}", TAG, json);
        } else if (rootNode.isArray() && !rootNode.isEmpty() && rootNode.get(0).isInt()) { // 处理数组类型
            List<Integer> customerIds = objectMapper.readValue(mqttMessage, new TypeReference<List<Integer>>() {
            });
            String json = objectMapper.writeValueAsString(customerIds);
            mqttOutboundChannel.send(MessageBuilder.withPayload(json).build());
            log.info("{} : -----> MQTT message = {}", TAG, json);
        } else {
            log.warn("收到未知格式的 MQTT 消息: {}", mqttMessage);
        }
    }
}